# File generated from our OpenAPI spec by Stainless.

from typing_extensions import Literal

__all__ = ["ExternalAccountType"]

ExternalAccountType = Literal["cash", "checking", "loan", "non_resident", "other", "overdraft", "savings"]
